%
%   Test_InvKin.M   This file provides a test case of the Inverse Kinetics Method
%
%   The basic test scenario is as follows:
%     1.  With a given rho(t), we first simulate P(t).
%     2.  With this P(t) we use the Inverse Kinetics method to calculate rho(t).  
%     3.  The test: if rho_calc is approximately the same as rho_applied.
%         then this should be a good test of the basic theory and actual 
%         implementation of the method
%
%   This simulation is generic, but the kinetics data used is specific to the UMLRR. 
%   Random noise in introduced into the simulated P(t) data to mimic what might be
%   measured in a real system...
%
%   Let's give this a try...
%   
%   File prepared J. R. White, UMass-Lowell (last update:  March 2018)
%

      clear all,  close all,  nfig = 0;  
%%
%   define UMLRR-specific kinetics data and other parameters for the simulations
      [Be,B,Gt,lam,td,S,kappa,nu] = kinetics_data();
      tol  = 1e-5;  options = odeset('RelTol',tol); % error tolerance for ODE solver
      ftime = 300;            % final simulation time (seconds)
      dt = 1.0;               % sampling time (sec)
      t = (0:dt:ftime)';      % time vector with sampling time set by dt
      nlevel = 0.05;          % random noise level (random uncertainty in P(t) data)
%%
%   do simulation with ICs at just critical and no source and given rho(t)
      tt =   [0   5  55 60 120  125  175 180 ftime];    
      rhot = [0 0.1 0.1  0   0 -0.1 -0.1   0   0]/100;  % rho(t) for step insertion 
      rho0 = 0;  Q = 0;       % critcal initial state  (i.e. rho = 0 and no source) 
      Po = 1;                 % initial power level (magnitude not important here)
      Co = B*Po./(lam*Gt);    % initial precursor conc
      xo = [Po Co]';          % initial condition for state vector
      ftx = @(t,x) pkeqns_nofdbk(t,x,rho0,Q,Be,Gt,B,lam,kappa,nu,tt,rhot);
      [t,x] = ode15s(ftx,t,xo,options);
      P = x(:,1)/Po;          % normalized P(t)
%
      nfig = nfig+1;   figure(nfig)
      subplot(2,1,1),plot(tt,rhot*100,'b-','LineWidth',2),grid
      title('Test\_InvKin:  Actual \rho_{ext}(t) and Clean Simulated P(t)')
      ylabel('Reactivity  (%\Deltak/k)')
      subplot(2,1,2),plot(t,P,'r-','LineWidth',2),grid
      xlabel('Time (sec)'),ylabel('Normalized Power')
%%
%   now with clean P(t), lets generate measured rho(t) with inverse kinetics
      rhoinvc = invkin_sr(P,t,lam,Co,dt,Be,Gt,B);
      nfig = nfig+1;   figure(nfig)
      plot(t,rhoinvc*100,'r-',tt,rhot*100,'b--','LineWidth',2),grid
      title('Test\_InvKin:  Comparison of Actual and Measured \rho_{ext}(t)')
      xlabel('Time (sec)'),ylabel('Reactivity  (%\Deltak/k)')
      legend('Computed \rho(t)','Input \rho(t)')
%%
%   add some noise to the P(t) signal
      rn = rand(size(t));     % vector of uniform random numbers (0 to 1)
      a = 1-nlevel;   b = 1+nlevel;  % nlevel is the 'noise level'  (see above)
      srn = (b-a)*rn + a;     % uniformly distributed between a and b
      srn(1) = 1.0;           % reset initial value to 1.0
      Pn = srn.*P;            % P(t) with random noise
%
%   plot noisy P(t) vs t
      nfig = nfig+1;   figure(nfig)
      subplot(2,1,1),plot(tt,rhot*100,'b-','LineWidth',2),grid
      title('Test\_InvKin:  Actual \rho_{ext}(t) and Noisy P(t)')
      ylabel('Reactivity  (%\Deltak/k)')
      subplot(2,1,2),ylabel('Applied \rho(t)  (%\Deltak/k)')
      plot(t,Pn,'r-','LineWidth',2),grid on
      xlabel('Time (sec)'),ylabel('Normalized Power')
%%
%   now with noisy P(t), lets generate measured rho(t) with inverse kinetics
      rhoinvn = invkin_sr(Pn,t,lam,Co,dt,Be,Gt,B);
      nfig = nfig+1;   figure(nfig)
      plot(t,rhoinvn*100,'r-',tt,rhot*100,'b--','LineWidth',2),grid
      title('Test\_InvKin:  Comparison of Actual and Measured \rho_{ext}(t)')
      xlabel('Time (sec)'),ylabel('Reactivity  (%\Deltak/k)')
      legend('Computed \rho(t)','Input \rho(t)')
%
%   end of program
